/*++
  The material contained herein is not a license, either        
  expressly or impliedly, to any intellectual property owned    
  or controlled by any of the authors or developers of this     
  material or to any contribution thereto. The material         
  contained herein is provided on an "AS IS" basis and, to the  
  maximum extent permitted by applicable law, this information  
  is provided AS IS AND WITH ALL FAULTS, and the authors and    
  developers of this material hereby disclaim all other         
  warranties and conditions, either express, implied or         
  statutory, including, but not limited to, any (if any)        
  implied warranties, duties or conditions of merchantability,  
  of fitness for a particular purpose, of accuracy or           
  completeness of responses, of results, of workmanlike         
  effort, of lack of viruses and of lack of negligence, all     
  with regard to this material and any contribution thereto.    
  Designers must not rely on the absence or characteristics of  
  any features or instructions marked "reserved" or             
  "undefined." The Unified EFI Forum, Inc. reserves any         
  features or instructions so marked for future definition and  
  shall have no responsibility whatsoever for conflicts or      
  incompatibilities arising from future changes to them. ALSO,  
  THERE IS NO WARRANTY OR CONDITION OF TITLE, QUIET ENJOYMENT,  
  QUIET POSSESSION, CORRESPONDENCE TO DESCRIPTION OR            
  NON-INFRINGEMENT WITH REGARD TO THE TEST SUITE AND ANY        
  CONTRIBUTION THERETO.                                         
                                                                
  IN NO EVENT WILL ANY AUTHOR OR DEVELOPER OF THIS MATERIAL OR  
  ANY CONTRIBUTION THERETO BE LIABLE TO ANY OTHER PARTY FOR     
  THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES, LOST      
  PROFITS, LOSS OF USE, LOSS OF DATA, OR ANY INCIDENTAL,        
  CONSEQUENTIAL, DIRECT, INDIRECT, OR SPECIAL DAMAGES WHETHER   
  UNDER CONTRACT, TORT, WARRANTY, OR OTHERWISE, ARISING IN ANY  
  WAY OUT OF THIS OR ANY OTHER AGREEMENT RELATING TO THIS       
  DOCUMENT, WHETHER OR NOT SUCH PARTY HAD ADVANCE NOTICE OF     
  THE POSSIBILITY OF SUCH DAMAGES.                              
                                                                
  Copyright 2006, 2007, 2008, 2009, 2010 Unified EFI, Inc. All  
  Rights Reserved, subject to all existing rights in all        
  matters included within this Test Suite, to which United      
  EFI, Inc. makes no claim of right.                            
                                                                
  Copyright (c) 2010, Intel Corporation. All rights reserved.   
   
--*/
/*++

Module Name:

  LibSalPal.h

Abstract:

  Function prototype definitions.

--*/

#ifndef _EFI_LIB_SAL_PAL_H_
#define _EFI_LIB_SAL_PAL_H_

#include "PalApi.h"
#include "SalApi.h"

typedef PAL_RETURN_REGS (EFIAPI *PAL_PROC) (IN UINT64 FunctionId, IN UINT64 Arg2, IN UINT64 Arg3, IN UINT64 Arg4);

EFI_STATUS
LibInitSalAndPalProc (
  VOID
  )
/*++

Routine Description:

  Initialize SAL and PAL procedure calling.

Arguments:

  None

Returns:

  EFI_SUCCESS - Always return success.

--*/
;

EFI_STATUS
LibGetSalVersionInfo (
  IN  OUT   UINT16  *SalSpecRev,
  IN  OUT   UINT16  *SalARev,
  IN  OUT   UINT16  *SalBRev
  )
/*++

Routine Description:

  Get SAL version infomation.

Arguments:

  SalSpecRev  - SAL Spec reversion.
  SalARev     - SAL A reversion.
  SalBRev     - SAL B reversion.

Returns:

  EFI_UNSUPPORTED - mSalSystemTable is NULL.
  EFI_SUCCESS - Operation succeeded.

--*/
;

EFI_STATUS
LibGetSalIoPortMapping (
  OUT UINT64  *IoPortMapping
  )
/*++

Routine Description:

  Get the IO Port Map from the SAL System Table.

Arguments:

  IoPortMapping  - Pointer to receive IO port mapping address.

Returns:

  EFI_UNSUPPORTED - mSalSystemTable is NULL.
  EFI_SUCCESS - Operation succeeded.

--*/
;

EFI_STATUS
LibGetSalIpiBlock (
  OUT UINT64  *IpiBlock
  )
/*++

Routine Description:

  Get the IPI block from the SAL system table

Arguments:

  IpiBlock  - Pointer to receive IPI block address.

Returns:

  EFI_UNSUPPORTED - SalMemDesc is NULL.
  EFI_SUCCESS - Operation succeeded.

--*/
;

EFI_STATUS
LibGetSalWakeupVector (
  OUT UINT64  *WakeVector
  )
/*++

Routine Description:

  Get the wakeup vector from the SAL system table

Arguments:

  WakeVector  - Pointer to receive wakeup vector.

Returns:

  EFI_UNSUPPORTED - Cannot find wakeup decriptor.
  EFI_SUCCESS - Operation succeeded.

--*/
;

VOID  *
LibSearchSalSystemTable (
  IN  UINT8   EntryType
  )
/*++

Routine Description:

  Search SAL system table.

Arguments:

  EntryType - SAL entry type.

Returns:

  None.

--*/
;

SAL_RETURN_REGS
LibSalProc (
  IN  UINT64    Arg1,
  IN  UINT64    Arg2,
  IN  UINT64    Arg3,
  IN  UINT64    Arg4,
  IN  UINT64    Arg5,
  IN  UINT64    Arg6,
  IN  UINT64    Arg7,
  IN  UINT64    Arg8
  )
;

#endif
